import json
from datetime import datetime


class Event:
    id: int
    event_name: str
    event_date: datetime

    def __init__(self, **kwargs):
        self.event_date = datetime.now()

        for key, value in kwargs.items():
            setattr(self, key, value)

    def dict(self):
        return {
            "event_name": self.event_name,
            "event_date": self.event_date,
        }

    def json(self):
        return json.dumps(self.dict())

    @classmethod
    def from_json(cls, event_data: str):
        data = json.loads(event_data)
        return cls(**data)


class AckEvent(Event):
    acknowledged: bool = False



from assimilator.database.specification import Specification


class FilterSpecification(Specification):
    def __init__(self, *filters, **filters_by):
        self.filters = filters
        self.filters_by = filters_by

    def apply(self, query):
        return query.filter(*self.filters).filter_by(**self.filters_by)


class ValueInSpecification(FilterSpecification):
    def __init__(self, field, values):
        super(ValueInSpecification, self).__init__(field.in_(values))



import assimilator.events
import assimilator.database
import assimilator.patterns
import assimilator.internal


try:
    import assimilator.alchemy
except (ImportError, ModuleNotFoundError):
    pass

try:
    import assimilator.kafka
except (ImportError, ModuleNotFoundError):
    pass



from abc import ABC, abstractmethod


class Specification(ABC):
    @abstractmethod
    def apply(self, query):
        raise NotImplementedError("Specification must specify apply()")

    def __call__(self, query):
        return self.apply(query)
